@@ -19,16 +19,36 @@
  */
 package com.ecyrd.jspwiki;
 
+/**
+ *  Defines a search result.
+ *  
+ *  @author Janne Jalkanen
+ */
+// FIXME3.0: Move to the search-package
 public interface SearchResult
 {
     /**
      *  Return the page.
+     *  
+     *  @return the WikiPage object containing this result
      */
     public WikiPage getPage();
 
     /**
      *  Returns the score.
+     *  
+     *  @return A positive score value.  Note that there is no upper limit for the score.
      */
 
     public int getScore();
+    
+    
+    /**
+     * Collection of XHTML fragments representing some contexts in which
+     * the match was made (a.k.a., "snippets").
+     *
+     * @return the search results
+     * @since 2.4
+     */
+    public String[] getContexts();
 }
