@@ -19,15 +19,17 @@
  */
 package com.ecyrd.jspwiki;
 
-import java.util.Properties;
-import java.util.Iterator;
+import java.security.Principal;
 import java.util.Date;
+import java.util.Iterator;
 import java.util.List;
-import org.apache.log4j.Logger;
+import java.util.Properties;
+
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
 
-import com.ecyrd.jspwiki.auth.UserProfile;
+import com.ecyrd.jspwiki.filters.PageFilter;
+import com.ecyrd.jspwiki.modules.InternalModule;
 
 /**
  *  Manages variables.  Variables are case-insensitive.  A list of all
@@ -38,8 +40,9 @@ import com.ecyrd.jspwiki.auth.UserProfile;
  */
 public class VariableManager
 {
-    private static Logger log = Logger.getLogger( VariableManager.class );
+    //private static Logger log = Logger.getLogger( VariableManager.class );
    
+    // FIXME: These are probably obsolete.
     public static final String VAR_ERROR = "error";
     public static final String VAR_MSG   = "msg";
     
@@ -48,10 +51,14 @@ public class VariableManager
      *  Put names here in lower case.
      */
     
-    protected static final String[] THE_BIG_NO_NO_LIST = {
+    static final String[] THE_BIG_NO_NO_LIST = {
         "jspwiki.auth.masterpassword"
     };
     
+    /**
+     *  Creates a VariableManager object using the property list given.
+     *  @param props The properties.
+     */
     public VariableManager( Properties props )
     {
     }
@@ -61,6 +68,9 @@ public class VariableManager
      *  a variable.
      *  <P>
      *  Currently we just check if the link starts with "{$".
+     *  
+     *  @param link The link text
+     *  @return true, if this represents a variable link.
      */
     public static boolean isVariableLink( String link )
     {
@@ -68,12 +78,16 @@ public class VariableManager
     }
 
     /**
-     *  Parses the link and finds a value.
-     *  <P>
-     *  A variable is inserted using the notation [{$variablename}].
+     *  Parses the link and finds a value.  This is essentially used
+     *  once {@link #isVariableLink(String)} has found that the link text
+     *  actually contains a variable.  For example, you could pass in
+     *  "{$username}" and get back "JanneJalkanen".
      *
+     *  @param  context The WikiContext
+     *  @param  link    The link text containing the variable name.
+     *  @return The variable value.
      *  @throws IllegalArgumentException If the format is not valid (does not 
-     *          start with {$, is zero length, etc.)
+     *          start with "{$", is zero length, etc.)
      *  @throws NoSuchVariableException If a variable is not known.
      */
     public String parseAndGetValue( WikiContext context,
@@ -102,6 +116,7 @@ public class VariableManager
      *
      *  @param context The WikiContext of the current page.
      *  @param source  The source string.
+     *  @return The source string with variables expanded.
      */
     // FIXME: somewhat slow.
     public String expandVariables( WikiContext context,
@@ -154,6 +169,16 @@ public class VariableManager
         return result.toString();
     }
 
+    /**
+     *  Returns the value of a named variable.  See {@link #getValue(WikiContext, String)}.
+     *  The only difference is that this method does not throw an exception, but it
+     *  returns the given default value instead.
+     *  
+     *  @param context WikiContext
+     *  @param varName The name of the variable
+     *  @param defValue A default value.
+     *  @return The variable value, or if not found, the default value.
+     */
     public String getValue( WikiContext context, String varName, String defValue )
     {
         try
@@ -167,10 +192,26 @@ public class VariableManager
     }
     
     /**
-     *  Returns a value of the named variable.
+     *  Returns a value of the named variable.  The resolving order is
+     *  <ol>
+     *    <li>Known "constant" name, such as "pagename", etc.  This is so
+     *        that pages could not override certain constants.
+     *    <li>WikiContext local variable.  This allows a programmer to
+     *        set a parameter which cannot be overridden by user.
+     *    <li>HTTP Session
+     *    <li>HTTP Request parameters
+     *    <li>WikiPage variable.  As set by the user with the SET directive.
+     *    <li>jspwiki.properties
+     *  </ol>
      *
+     *  Use this method only whenever you really need to have a parameter that
+     *  can be overridden by anyone using the wiki.
+     *  
+     *  @param context The WikiContext
      *  @param varName Name of the variable.
      *
+     *  @return The variable value.
+     *  
      *  @throws IllegalArgumentException If the name is somehow broken.
      *  @throws NoSuchVariableException If a variable is not known.
      */
@@ -189,7 +230,6 @@ public class VariableManager
 
         // Faster than doing equalsIgnoreCase()
         String name = varName.toLowerCase();
-        String res = "";
 
         for( int i = 0; i < THE_BIG_NO_NO_LIST.length; i++ )
         {
@@ -199,70 +239,80 @@ public class VariableManager
         
         if( name.equals("pagename") )
         {
-            res = context.getPage().getName();
+            return context.getPage().getName();
         }
         else if( name.equals("applicationname") )
         {
-            res = context.getEngine().getApplicationName();
+            return context.getEngine().getApplicationName();
         }
         else if( name.equals("jspwikiversion") )
         {
-            res = Release.getVersionString();
+            return Release.getVersionString();
         }
         else if( name.equals("encoding") )
         {
-            res = context.getEngine().getContentEncoding();
+            return context.getEngine().getContentEncoding();
         }
         else if( name.equals("totalpages") )
         {
-            res = Integer.toString(context.getEngine().getPageCount());
+            return Integer.toString(context.getEngine().getPageCount());
         }
         else if( name.equals("pageprovider") )
         {
-            res = context.getEngine().getCurrentProvider();
+            return context.getEngine().getCurrentProvider();
         }
         else if( name.equals("pageproviderdescription") )
         {
-            res = context.getEngine().getCurrentProviderInfo();
+            return context.getEngine().getCurrentProviderInfo();
         }
         else if( name.equals("attachmentprovider") )
         {
             WikiProvider p = context.getEngine().getAttachmentManager().getCurrentProvider();
-            res = (p != null) ? p.getClass().getName() : "-";
+            return (p != null) ? p.getClass().getName() : "-";
         }
         else if( name.equals("attachmentproviderdescription") )
         {
             WikiProvider p = context.getEngine().getAttachmentManager().getCurrentProvider();
 
-            res = (p != null) ? p.getProviderInfo() : "-";
+            return (p != null) ? p.getProviderInfo() : "-";
         }
         else if( name.equals("interwikilinks") )
         {
-            // FIXME: Use StringBuffer
+            StringBuffer res = new StringBuffer();
+
             for( Iterator i = context.getEngine().getAllInterWikiLinks().iterator(); i.hasNext(); )
             {
+                if( res.length() > 0 ) res.append(", ");
                 String link = (String) i.next();
-                res += link+" --&gt; "+context.getEngine().getInterWikiURL(link)+"<br />\n";
+                res.append( link );
+                res.append( " --> " );
+                res.append( context.getEngine().getInterWikiURL(link) );    
             }
+            return res.toString();
         }
         else if( name.equals("inlinedimages") )
         {
-            // FIXME: Use StringBuffer
+            StringBuffer res = new StringBuffer();
+
             for( Iterator i = context.getEngine().getAllInlinedImagePatterns().iterator(); i.hasNext(); )
             {
+                if( res.length() > 0 ) res.append(", ");
+                
                 String ptrn = (String) i.next();
-                res += ptrn+"<br />\n";
+                res.append(ptrn);
             }
+            
+            return res.toString();
         }
         else if( name.equals("pluginpath") )
         {
-            res = context.getEngine().getPluginSearchPath();
+            String s = context.getEngine().getPluginSearchPath();
 
-            res = (res == null) ? "-" : res;
+            return (s == null) ? "-" : s;
         }
         else if( name.equals("baseurl") )
         {
-            res = context.getEngine().getBaseURL();
+            return context.getEngine().getBaseURL();
         }
         else if( name.equals("uptime") )
         {
@@ -278,27 +328,12 @@ public class VariableManager
         }
         else if( name.equals("loginstatus") )
         {
-            UserProfile wup = context.getCurrentUser();
-
-            int status = (wup != null) ? wup.getLoginStatus() : UserProfile.NONE;
-
-            switch( status )
-            {
-              case UserProfile.NONE:
-                return "unknown (not logged in)";
-              case UserProfile.COOKIE:
-                return "named (cookie)";
-              case UserProfile.CONTAINER:
-                return "validated (container)";
-              case UserProfile.PASSWORD:
-                return "validated (password)";
-              default:
-                return "ILLEGAL STATUS!";
-            }
+            WikiSession session = context.getWikiSession();
+            return session.getStatus();
         }
         else if( name.equals("username") )
         {
-            UserProfile wup = context.getCurrentUser();
+            Principal wup = context.getCurrentUser();
 
             return wup != null ? wup.getName() : "not logged in";
         }
@@ -313,14 +348,10 @@ public class VariableManager
 
             for( Iterator i = filters.iterator(); i.hasNext(); )
             {
-                String f = i.next().getClass().getName();
+                PageFilter pf = (PageFilter)i.next();
+                String f = pf.getClass().getName();
 
-                //
-                //  Skip some known internal filters.
-                //  FIXME: Quite a klugde.
-                //
-
-                if( f.endsWith("ReferenceManager") || f.endsWith("WikiDatabase$SaveFilter" ) )
+                if( pf instanceof InternalModule )
                     continue;
 
                 if( sb.length() > 0 ) sb.append(", ");
@@ -340,6 +371,39 @@ public class VariableManager
                 return context.getVariable( varName ).toString();
             }
 
+            //
+            //  Well, I guess it wasn't a final straw.  We also allow 
+            //  variables from the session and the request (in this order).
+            //
+
+            HttpServletRequest req = context.getHttpRequest();
+            if( req != null && req.getSession() != null )
+            {
+                HttpSession session = req.getSession();
+
+                try
+                {
+                    String s;
+                    
+                    if( (s = (String)session.getAttribute( varName )) != null )
+                        return s;
+
+                    if( (s = context.getHttpParameter( varName )) != null )
+                        return s;
+                }
+                catch( ClassCastException e ) {}
+            }
+
+            // And the final straw: see if the current page has named metadata.
+            
+            WikiPage pg = context.getPage();
+            if( pg != null )
+            {
+                Object metadata = pg.getAttribute( varName );
+                if( metadata != null )
+                    return metadata.toString();
+            }
+
             // Next-to-final straw: attempt to fetch using property name
             // We don't allow fetching any other properties than those starting
             // with "jspwiki.".  I know my own code, but I can't vouch for bugs
@@ -349,44 +413,13 @@ public class VariableManager
             {
                 Properties props = context.getEngine().getWikiProperties();
 
-                res = props.getProperty( varName );
-                if( res != null )
+                String s = props.getProperty( varName );
+                if( s != null )
                 {
-                    return res;
+                    return s;
                 }
             }
             
-            // And the final straw: see if the current page has named metadata.
-            
-            WikiPage pg = context.getPage();
-            if( pg != null )
-            {
-                Object metadata = pg.getAttribute( varName );
-                if( metadata != null )
-                    return( metadata.toString() );
-            }
-
-            //
-            //  Well, I guess it wasn't a final straw.  We also allow 
-            //  variables from the session and the request (in this order).
-            //
-
-            HttpServletRequest req = context.getHttpRequest();
-            if( req != null )
-            {
-                HttpSession session = req.getSession();
-
-                try
-                {
-                    if( (res = (String)session.getAttribute( varName )) != null )
-                        return res;
-
-                    if( (res = context.getHttpParameter( varName )) != null )
-                        return res;
-                }
-                catch( ClassCastException e ) {}
-            }
-
             //
             //  Final defaults for some known quantities.
             //
@@ -396,7 +429,5 @@ public class VariableManager
   
             throw new NoSuchVariableException( "No variable "+varName+" defined." );
         }
-        
-        return res;
     }
 }
