@@ -21,10 +21,18 @@ package com.ecyrd.jspwiki.auth;
 
 /**
  *  Thrown in some error situations where a WikiPrincipal object does not exist.
+ *  @author Andrew Jaquith
+ *  @since 2.3
  */
-public class NoSuchPrincipalException
+public final class NoSuchPrincipalException
     extends WikiSecurityException
 {
+    private static final long serialVersionUID = 3257849895976186169L;
+
+    /**
+     * Constructs a new exception object with a supplied message.
+     * @param msg the message
+     */
     public NoSuchPrincipalException( String msg )
     {
         super(msg);
