@@ -26,10 +26,10 @@ import java.text.SimpleDateFormat;
 
 /**
  *  Just displays the current date and time.
- *  The time format is exactly like in the java.util.SimpleDateFormat class.
+ *  The time format is exactly like in the java.text.SimpleDateFormat class.
  *
  *  @since 1.7.8
- *  @see java.util.SimpleDateFormat
+ *  @see java.text.SimpleDateFormat
  *  @author Janne Jalkanen
  */
 public class CurrentTimePlugin
@@ -61,7 +61,9 @@ public class CurrentTimePlugin
         }
         catch( IllegalArgumentException e )
         {
-            throw new PluginException("You specified bad format: "+e.getMessage());
+            ResourceBundle rb = context.getBundle(WikiPlugin.CORE_PLUGINS_RESOURCEBUNDLE);
+            
+            throw new PluginException( rb.getString("currenttimeplugin.badformat") + e.getMessage() );
         }
     }
 
