@@ -35,8 +35,10 @@ import java.util.Iterator;
 import java.util.Date;
 import java.util.List;
 import java.util.Collections;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
-import org.apache.log4j.Category;
+import org.apache.log4j.Logger;
 
 import com.ecyrd.jspwiki.*;
 import com.ecyrd.jspwiki.attachment.Attachment;
@@ -74,21 +76,36 @@ import com.ecyrd.jspwiki.attachment.Attachment;
 public class BasicAttachmentProvider
     implements WikiAttachmentProvider
 {
-    private String m_storageDir;
+    private WikiEngine         m_engine;
+    private String             m_storageDir;
     public static final String PROP_STORAGEDIR = "jspwiki.basicAttachmentProvider.storageDir";
+    
+    /*
+     * Disable client cache for files with patterns
+     * since 2.5.96
+     */
+    private Pattern            m_disableCache = null;
+    public static final String PROP_DISABLECACHE = "jspwiki.basicAttachmentProvider.disableCache";
 
     public static final String PROPERTY_FILE   = "attachment.properties";
 
     public static final String DIR_EXTENSION   = "-att";
     public static final String ATTDIR_EXTENSION = "-dir";
     
-    static final Category log = Category.getInstance( BasicAttachmentProvider.class );
+    static final Logger log = Logger.getLogger( BasicAttachmentProvider.class );
 
-    public void initialize( Properties properties ) 
+    public void initialize( WikiEngine engine, Properties properties ) 
         throws NoRequiredPropertyException,
                IOException
     {
+        m_engine = engine;
         m_storageDir = WikiEngine.getRequiredProperty( properties, PROP_STORAGEDIR );
+        
+        String patternString = engine.getWikiProperties().getProperty( PROP_DISABLECACHE );
+        if ( patternString != null )
+        {
+            m_disableCache = Pattern.compile(patternString);
+        }
 
         //
         //  Check if the directory exists - if it doesn't, create it.
@@ -99,6 +116,18 @@ public class BasicAttachmentProvider
         {
             f.mkdirs();
         }
+
+        //
+        // Some sanity checks
+        //
+        if( !f.exists() ) 
+            throw new IOException("Could not find or create attachment storage directory '"+m_storageDir+"'");
+
+        if( !f.canWrite() ) 
+            throw new IOException("Cannot write to the attachment storage directory '"+m_storageDir+"'");
+        
+        if( !f.isDirectory() )
+            throw new IOException("Your attachment storage points to a file, not a directory: '"+m_storageDir+"'");
     }
 
     /**
@@ -265,7 +294,7 @@ public class BasicAttachmentProvider
 
         File propertyFile = new File( findAttachmentDir(att), PROPERTY_FILE );
 
-        if( propertyFile != null && propertyFile.exists() )
+        if( propertyFile.exists() )
         {
             InputStream in = new FileInputStream( propertyFile );
 
@@ -318,6 +347,13 @@ public class BasicAttachmentProvider
             }
 
             props.setProperty( versionNumber+".author", author );
+            
+            String changeNote = (String)att.getAttribute(WikiPage.CHANGENOTE);
+            if( changeNote != null )
+            {
+                props.setProperty( versionNumber+".changenote", changeNote );
+            }
+            
             putPageProperties( att, props );
         }
         catch( IOException e )
@@ -327,7 +363,7 @@ public class BasicAttachmentProvider
         }
         finally
         {
-                if( out != null ) out.close();
+            if( out != null ) out.close();
         }
     }
 
@@ -373,9 +409,17 @@ public class BasicAttachmentProvider
     {
         File attDir = findAttachmentDir( att );
 
-        File f = findFile( attDir, att );
+        try
+        {
+            File f = findFile( attDir, att );
 
-        return new FileInputStream( f );
+            return new FileInputStream( f );
+        }
+        catch( FileNotFoundException e )
+        {
+            log.error("File not found: "+e.getMessage());
+            throw new ProviderException("No such page was found.");
+        }
     }
 
     public Collection listAttachments( WikiPage page )
@@ -391,6 +435,10 @@ public class BasicAttachmentProvider
 
             if( attachments != null )
             {
+                //
+                //  We now have a list of all potential attachments in 
+                //  the directory.
+                //
                 for( int i = 0; i < attachments.length; i++ )
                 {
                     File f = new File( dir, attachments[i] );
@@ -399,14 +447,37 @@ public class BasicAttachmentProvider
                     {
                         String attachmentName = unmangleName( attachments[i] );
 
+                        //
+                        //  Is it a new-stylea attachment directory?  If yes,
+                        //  we'll just deduce the name.  If not, however,
+                        //  we'll check if there's a suitable property file
+                        //  in the directory.
+                        //
                         if( attachmentName.endsWith( ATTDIR_EXTENSION ) )
                         {
                             attachmentName = attachmentName.substring( 0, attachmentName.length()-ATTDIR_EXTENSION.length() );
                         }
-                        
+                        else
+                        {
+                            File propFile = new File( f, PROPERTY_FILE );
+
+                            if( !propFile.exists() )
+                            {
+                                //
+                                //  This is not obviously a JSPWiki attachment,
+                                //  so let's just skip it.
+                                //
+                                continue;
+                            }
+                        }
+
                         Attachment att = getAttachmentInfo( page, attachmentName,
                                                             WikiProvider.LATEST_VERSION );
 
+                        //
+                        //  Sanity check - shouldn't really be happening, unless
+                        //  you mess with the repository directly.
+                        //
                         if( att == null )
                         {
                             throw new ProviderException("Attachment disappeared while reading information:"+
@@ -449,7 +520,7 @@ public class BasicAttachmentProvider
             String pageId = unmangleName( pagesWithAttachments[i] );
             pageId = pageId.substring( 0, pageId.length()-DIR_EXTENSION.length() );
             
-            Collection c = listAttachments( new WikiPage(pageId) );
+            Collection c = listAttachments( new WikiPage( m_engine, pageId ) );
 
             for( Iterator it = c.iterator(); it.hasNext(); )
             {
@@ -470,7 +541,7 @@ public class BasicAttachmentProvider
     public Attachment getAttachmentInfo( WikiPage page, String name, int version )
         throws ProviderException
     {
-        Attachment att = new Attachment( page.getName(), name );
+        Attachment att = new Attachment( m_engine, page.getName(), name );
         File dir = findAttachmentDir( att );
 
         if( !dir.exists() )
@@ -486,6 +557,16 @@ public class BasicAttachmentProvider
 
         att.setVersion( version );
         
+        // Should attachment be cachable by the client (browser)?
+        if (m_disableCache != null) 
+        {
+            Matcher matcher = m_disableCache.matcher(name);
+            if (matcher.matches()) 
+            {
+                att.setCacheable(false);
+            }
+        }
+        
 
         // System.out.println("Fetching info on version "+version);
         try
@@ -494,11 +575,21 @@ public class BasicAttachmentProvider
 
             att.setAuthor( props.getProperty( version+".author" ) );
 
+            String changeNote = props.getProperty( version+".changenote" );
+            if( changeNote != null )
+            {
+                att.setAttribute(WikiPage.CHANGENOTE, changeNote);
+            }
+            
             File f = findFile( dir, att );
 
             att.setSize( f.length() );
             att.setLastModified( new Date(f.lastModified()) );
         }
+        catch( FileNotFoundException e )
+        {
+            return null;
+        }
         catch( IOException e )
         {
             log.error("Can't read page properties", e );
@@ -519,7 +610,7 @@ public class BasicAttachmentProvider
 
             for( int i = latest; i >= 1; i-- )
             {
-                Attachment a = getAttachmentInfo( new WikiPage(att.getParentName()), 
+                Attachment a = getAttachmentInfo( new WikiPage( m_engine, att.getParentName() ), 
                                                   att.getFileName(), i );
 
                 if( a != null )
@@ -547,14 +638,22 @@ public class BasicAttachmentProvider
     public void deleteAttachment( Attachment att )
         throws ProviderException
     {
-        // FIXME: Does nothing yet.
+        File dir = findAttachmentDir( att );
+        String[] files = dir.list();
+
+        for( int i = 0; i < files.length; i++ )
+        {
+            File file = new File( dir.getAbsolutePath() + "/" + files[i] );
+            file.delete();
+        }
+        dir.delete();
     }
 
 
     /**
      *  Returns only those directories that contain attachments.
      */
-    public class AttachmentFilter
+    public static class AttachmentFilter
         implements FilenameFilter
     {
         public boolean accept( File dir, String name )
@@ -566,7 +665,7 @@ public class BasicAttachmentProvider
     /**
      *  Accepts only files that are actual versions, no control files.
      */
-    public class AttachmentVersionFilter
+    public static class AttachmentVersionFilter
         implements FilenameFilter
     {
         public boolean accept( File dir, String name )
@@ -575,5 +674,25 @@ public class BasicAttachmentProvider
         }
     }
 
+    public void moveAttachmentsForPage( String oldParent, String newParent )
+        throws ProviderException
+    {
+        File srcDir = findPageDir( oldParent );
+        File destDir = findPageDir( newParent );
+
+        log.debug("Trying to move all attachments from "+srcDir+" to "+destDir);
+
+        // If it exists, we're overwriting an old page (this has already been
+        // confirmed at a higher level), so delete any existing attachments.
+        if (destDir.exists())
+        {
+            log.error("Page rename failed because target dirctory "+destDir+" exists");
+        }
+        else
+        {
+            //destDir.getParentFile().mkdir();
+            srcDir.renameTo(destDir);
+        }
+    }
 }
 
