@@ -47,6 +47,8 @@ import com.ecyrd.jspwiki.providers.ProviderException;
 public class HistoryIteratorTag
     extends IteratorTag
 {
+    private static final long serialVersionUID = 0L;
+    
     static    Logger    log = Logger.getLogger( HistoryIteratorTag.class );
 
     public final int doStartTag()
@@ -89,7 +91,7 @@ public class HistoryIteratorTag
                 }
             }
 
-            return EVAL_BODY_TAG;
+            return EVAL_BODY_BUFFERED;
         }
         catch( ProviderException e )
         {
@@ -126,11 +128,9 @@ public class HistoryIteratorTag
                                       PageContext.REQUEST_SCOPE );
             pageContext.setAttribute( getId(),
                                       context.getPage() );
-            return EVAL_BODY_TAG;
-        }
-        else
-        {
-            return SKIP_BODY;
+            return EVAL_BODY_BUFFERED;
         }
+
+        return SKIP_BODY;
     }
 }
